@@ -682,7 +682,7 @@ namespace Ogre
         const RenderSystemCapabilities *capabilities = renderSystem->getCapabilities();
 
         //VTF must be supported
-        if( capabilities->hasCapability( RSC_VERTEX_TEXTURE_FETCH ) )
+        if( capabilities->getNumVertexTextureUnits() )
         {
             //TODO: Check PF_FLOAT32_RGBA is supported (should be, since it was the 1st one)
             const size_t numBones = std::max<size_t>( 1, baseSubMesh->blendIndexToBoneIndexMap.size() );
