@@ -28,6 +28,7 @@ THE SOFTWARE.
 
 
 #include "OgreXMLMeshSerializer.h"
+#include "OgreBitwise.h"
 #include "OgreSubMesh.h"
 #include "OgreLogManager.h"
 #include "OgreSkeleton.h"
@@ -44,6 +45,7 @@ THE SOFTWARE.
 #include <cstddef>
 
 namespace Ogre {
+    static const char* texcoordDimName[] = {"u", "v", "w", "x"};
 
     //---------------------------------------------------------------------
     XMLMeshSerializer::XMLMeshSerializer()
@@ -460,6 +462,18 @@ namespace Ogre {
                         case VET_UBYTE4: 
                             type = "ubyte4"; 
                             break;
+                        case VET_HALF1:
+                            type = "half1";
+                            break;
+                        case VET_HALF2:
+                            type = "half2";
+                            break;
+                        case VET_HALF3:
+                            type = "half3";
+                            break;
+                        case VET_HALF4:
+                            type = "half4";
+                            break;
                         default:
                             OgreAssert(false, "Unsupported VET");
                             break;
@@ -488,14 +502,25 @@ namespace Ogre {
                 for (i = elems.begin(); i != iend; ++i)
                 {
                     VertexElement& elem = *i;
+                    auto dims = VertexElement::getTypeCount(elem.getType());
                     switch(elem.getSemantic())
                     {
                     case VES_POSITION:
-                        elem.baseVertexPointerToElement(pVert, &pFloat);
                         dataNode = vertexNode.append_child("position");
-                        dataNode.append_attribute("x") = StringConverter::toString(pFloat[0]).c_str();
-                        dataNode.append_attribute("y") = StringConverter::toString(pFloat[1]).c_str();
-                        dataNode.append_attribute("z") = StringConverter::toString(pFloat[2]).c_str();
+                        if(elem.getType() == VET_HALF3)
+                        {
+                            elem.baseVertexPointerToElement(pVert, &pShort);
+                            dataNode.append_attribute("x") = StringConverter::toString(Bitwise::halfToFloat(*pShort++)).c_str();
+                            dataNode.append_attribute("y") = StringConverter::toString(Bitwise::halfToFloat(*pShort++)).c_str();
+                            dataNode.append_attribute("z") = StringConverter::toString(Bitwise::halfToFloat(*pShort++)).c_str();
+                        }
+                        else
+                        {
+                            elem.baseVertexPointerToElement(pVert, &pFloat);
+                            dataNode.append_attribute("x") = StringConverter::toString(pFloat[0]).c_str();
+                            dataNode.append_attribute("y") = StringConverter::toString(pFloat[1]).c_str();
+                            dataNode.append_attribute("z") = StringConverter::toString(pFloat[2]).c_str();
+                        }
                         break;
                     case VES_NORMAL:
                         elem.baseVertexPointerToElement(pVert, &pFloat);
@@ -543,51 +568,28 @@ namespace Ogre {
                     case VES_TEXTURE_COORDINATES:
                         dataNode = vertexNode.append_child("texcoord");
 
-                        switch(elem.getType())
+                        switch(VertexElement::getBaseType(elem.getType()))
                         {
                         case VET_FLOAT1:
                             elem.baseVertexPointerToElement(pVert, &pFloat);
-                            dataNode.append_attribute("u") = StringConverter::toString(*pFloat++).c_str();
-                            break;
-                        case VET_FLOAT2:
-                            elem.baseVertexPointerToElement(pVert, &pFloat);
-                            dataNode.append_attribute("u") = StringConverter::toString(*pFloat++).c_str();
-                            dataNode.append_attribute("v") = StringConverter::toString(*pFloat++).c_str();
-                            break;
-                        case VET_FLOAT3:
-                            elem.baseVertexPointerToElement(pVert, &pFloat);
-                            dataNode.append_attribute("u") = StringConverter::toString(*pFloat++).c_str();
-                            dataNode.append_attribute("v") = StringConverter::toString(*pFloat++).c_str();
-                            dataNode.append_attribute("w") = StringConverter::toString(*pFloat++).c_str();
-                            break;
-                        case VET_FLOAT4:
-                            elem.baseVertexPointerToElement(pVert, &pFloat);
-                            dataNode.append_attribute("u") = StringConverter::toString(*pFloat++).c_str();
-                            dataNode.append_attribute("v") = StringConverter::toString(*pFloat++).c_str();
-                            dataNode.append_attribute("w") = StringConverter::toString(*pFloat++).c_str();
-                            dataNode.append_attribute("x") = StringConverter::toString(*pFloat++).c_str();
+                            for(int j = 0; j < dims; ++j)
+                            {
+                                dataNode.append_attribute(texcoordDimName[j]) = StringConverter::toString(*pFloat++).c_str();
+                            }
                             break;
                         case VET_SHORT1:
                             elem.baseVertexPointerToElement(pVert, &pShort);
-                            dataNode.append_attribute("u") =  StringConverter::toString(*pShort++ / 65535.0f).c_str();
+                            for(int j = 0; j < dims; ++j)
+                            {
+                                dataNode.append_attribute(texcoordDimName[j]) = StringConverter::toString(*pShort++ / 65535.0f).c_str();
+                            }
                             break;
-                        case VET_SHORT2:
+                        case VET_HALF1:
                             elem.baseVertexPointerToElement(pVert, &pShort);
-                            dataNode.append_attribute("u") =  StringConverter::toString(*pShort++ / 65535.0f).c_str();
-                            dataNode.append_attribute("v") =  StringConverter::toString(*pShort++ / 65535.0f).c_str();
-                            break;
-                        case VET_SHORT3:
-                            elem.baseVertexPointerToElement(pVert, &pShort);
-                            dataNode.append_attribute("u") =  StringConverter::toString(*pShort++ / 65535.0f).c_str();
-                            dataNode.append_attribute("v") =  StringConverter::toString(*pShort++ / 65535.0f).c_str();
-                            dataNode.append_attribute("w") =  StringConverter::toString(*pShort++ / 65535.0f).c_str();
-                            break;
-                        case VET_SHORT4:
-                            elem.baseVertexPointerToElement(pVert, &pShort);
-                            dataNode.append_attribute("u") =  StringConverter::toString(*pShort++ / 65535.0f).c_str();
-                            dataNode.append_attribute("v") =  StringConverter::toString(*pShort++ / 65535.0f).c_str();
-                            dataNode.append_attribute("w") =  StringConverter::toString(*pShort++ / 65535.0f).c_str();
-                            dataNode.append_attribute("x") =  StringConverter::toString(*pShort++ / 65535.0f).c_str();
+                            for(int j = 0; j < dims; ++j)
+                            {
+                                dataNode.append_attribute(texcoordDimName[j]) = StringConverter::toString(Bitwise::halfToFloat(*pShort++)).c_str();
+                            }
                             break;
                         case VET_UBYTE4_NORM:
                             elem.baseVertexPointerToElement(pVert, &pColour);
@@ -599,10 +601,10 @@ namespace Ogre {
                             break;
                         case VET_UBYTE4:
                             elem.baseVertexPointerToElement(pVert, &pChar);
-                            dataNode.append_attribute("u") =  StringConverter::toString(*pChar++ / 255.0f).c_str();
-                            dataNode.append_attribute("v") =  StringConverter::toString(*pChar++ / 255.0f).c_str();
-                            dataNode.append_attribute("w") =  StringConverter::toString(*pChar++ / 255.0f).c_str();
-                            dataNode.append_attribute("x") =  StringConverter::toString(*pChar++ / 255.0f).c_str();
+                            for(int j = 0; j < 4; ++j)
+                            {
+                                dataNode.append_attribute(texcoordDimName[j]) = StringConverter::toString(*pChar++ / 255.0f).c_str();
+                            }
                             break;
                         default:
                             OgreAssert(false, "Unsupported VET");
@@ -940,6 +942,14 @@ namespace Ogre {
                             vtype = _DETAIL_SWAP_RB;
                         else if (!::strcmp(attrib,"colour_abgr"))
                             vtype = VET_UBYTE4_NORM;
+                        else if (!::strcmp(attrib,"half1"))
+                            vtype = VET_HALF1;
+                        else if (!::strcmp(attrib,"half2"))
+                            vtype = VET_HALF2;
+                        else if (!::strcmp(attrib,"half3"))
+                            vtype = VET_HALF3;
+                        else if (!::strcmp(attrib,"half4"))
+                            vtype = VET_HALF4;
                         else 
                         {
                             auto err = LogManager::getSingleton().stream(LML_CRITICAL);
@@ -985,6 +995,7 @@ namespace Ogre {
                 for (ielem = elems.begin(); ielem != ielemend; ++ielem)
                 {
                     const VertexElement& elem = *ielem;
+                    auto dims = VertexElement::getTypeCount(elem.getType());
                     // Find child for this element
                     switch(elem.getSemantic())
                     {
@@ -1106,103 +1117,43 @@ namespace Ogre {
                         // Record the latest texture coord entry
                         texCoordElem = xmlElem;
 
-                        if (!xmlElem.attribute("u"))
-                            OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'u' attribute not found.", "XMLMeshSerializer::readGeometry");
-                        
-                        // depending on type, pack appropriately, can process colour channels separately which is a bonus
-                        switch (elem.getType()) 
+                        for(int j = 0; j < dims; ++j)
+                        {
+                            if (!xmlElem.attribute(texcoordDimName[j]))
+                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, StringUtil::format("Texcoord attribute '%s' not found.", texcoordDimName[j]));
+                        }
+
+                        switch(VertexElement::getBaseType(elem.getType()))
                         {
                         case VET_FLOAT1:
                             elem.baseVertexPointerToElement(pVert, &pFloat);
-                            *pFloat++ = StringConverter::parseReal(xmlElem.attribute("u").value());
+                            for(int j = 0; j < dims; ++j)
+                            {
+                                *pFloat++ = StringConverter::parseReal(xmlElem.attribute(texcoordDimName[j]).value());
+                            }
                             break;
-
-                        case VET_FLOAT2:
-                            if (!xmlElem.attribute("v"))
-                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'v' attribute not found.", "XMLMeshSerializer::readGeometry");
-                            elem.baseVertexPointerToElement(pVert, &pFloat);
-                            *pFloat++ = StringConverter::parseReal(xmlElem.attribute("u").value());
-                            *pFloat++ = StringConverter::parseReal(xmlElem.attribute("v").value());
-                            break;
-
-                        case VET_FLOAT3:
-                            if (!xmlElem.attribute("v"))
-                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'v' attribute not found.", "XMLMeshSerializer::readGeometry");
-                            if (!xmlElem.attribute("w"))
-                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'w' attribute not found.", "XMLMeshSerializer::readGeometry");
-                            elem.baseVertexPointerToElement(pVert, &pFloat);
-                            *pFloat++ = StringConverter::parseReal(xmlElem.attribute("u").value());
-                            *pFloat++ = StringConverter::parseReal(xmlElem.attribute("v").value());
-                            *pFloat++ = StringConverter::parseReal(xmlElem.attribute("w").value());
-                            break;
-
-                        case VET_FLOAT4:
-                            if (!xmlElem.attribute("v"))
-                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'v' attribute not found.", "XMLMeshSerializer::readGeometry");
-                            if (!xmlElem.attribute("w"))
-                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'w' attribute not found.", "XMLMeshSerializer::readGeometry");
-                            if (!xmlElem.attribute("x"))
-                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'x' attribute not found.", "XMLMeshSerializer::readGeometry");
-                            elem.baseVertexPointerToElement(pVert, &pFloat);
-                            *pFloat++ = StringConverter::parseReal(xmlElem.attribute("u").value());
-                            *pFloat++ = StringConverter::parseReal(xmlElem.attribute("v").value());
-                            *pFloat++ = StringConverter::parseReal(xmlElem.attribute("w").value());
-                            *pFloat++ = StringConverter::parseReal(xmlElem.attribute("x").value());
-                            break;
-
                         case VET_SHORT1:
                             elem.baseVertexPointerToElement(pVert, &pShort);
-                            *pShort++ = static_cast<uint16>(65535.0f * StringConverter::parseReal(xmlElem.attribute("u").value()));
+                            for(int j = 0; j < dims; ++j)
+                            {
+                                *pShort++ = static_cast<uint16>(65535.0f * StringConverter::parseReal(xmlElem.attribute(texcoordDimName[j]).value()));
+                            }
                             break;
-
-                        case VET_SHORT2:
-                            if (!xmlElem.attribute("v"))
-                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'v' attribute not found.", "XMLMeshSerializer::readGeometry");
+                        case VET_HALF1:
                             elem.baseVertexPointerToElement(pVert, &pShort);
-                            *pShort++ = static_cast<uint16>(65535.0f * StringConverter::parseReal(xmlElem.attribute("u").value()));
-                            *pShort++ = static_cast<uint16>(65535.0f * StringConverter::parseReal(xmlElem.attribute("v").value()));
+                            for(int j = 0; j < dims; ++j)
+                            {
+                                *pShort++ = Bitwise::floatToHalf(StringConverter::parseReal(xmlElem.attribute(texcoordDimName[j]).value()));
+                            }
                             break;
-
-                        case VET_SHORT3:
-                            if (!xmlElem.attribute("v"))
-                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'v' attribute not found.", "XMLMeshSerializer::readGeometry");
-                            if (!xmlElem.attribute("w"))
-                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'w' attribute not found.", "XMLMeshSerializer::readGeometry");
-                            elem.baseVertexPointerToElement(pVert, &pShort);
-                            *pShort++ = static_cast<uint16>(65535.0f * StringConverter::parseReal(xmlElem.attribute("u").value()));
-                            *pShort++ = static_cast<uint16>(65535.0f * StringConverter::parseReal(xmlElem.attribute("v").value()));
-                            *pShort++ = static_cast<uint16>(65535.0f * StringConverter::parseReal(xmlElem.attribute("w").value()));
-                            break;
-
-                        case VET_SHORT4:
-                            if (!xmlElem.attribute("v"))
-                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'v' attribute not found.", "XMLMeshSerializer::readGeometry");
-                            if (!xmlElem.attribute("w"))
-                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'w' attribute not found.", "XMLMeshSerializer::readGeometry");
-                            if (!xmlElem.attribute("x"))
-                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'x' attribute not found.", "XMLMeshSerializer::readGeometry");
-                            elem.baseVertexPointerToElement(pVert, &pShort);
-                            *pShort++ = static_cast<uint16>(65535.0f * StringConverter::parseReal(xmlElem.attribute("u").value()));
-                            *pShort++ = static_cast<uint16>(65535.0f * StringConverter::parseReal(xmlElem.attribute("v").value()));
-                            *pShort++ = static_cast<uint16>(65535.0f * StringConverter::parseReal(xmlElem.attribute("w").value()));
-                            *pShort++ = static_cast<uint16>(65535.0f * StringConverter::parseReal(xmlElem.attribute("x").value()));
-                            break;
-
                         case VET_UBYTE4:
-                            if (!xmlElem.attribute("v"))
-                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'v' attribute not found.", "XMLMeshSerializer::readGeometry");
-                            if (!xmlElem.attribute("w"))
-                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'w' attribute not found.", "XMLMeshSerializer::readGeometry");
-                            if (!xmlElem.attribute("x"))
-                                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Texcoord 'x' attribute not found.", "XMLMeshSerializer::readGeometry");
                             elem.baseVertexPointerToElement(pVert, &pChar);
-                            // round off instead of just truncating -- avoids magnifying rounding errors
-                            *pChar++ = static_cast<uint8>(0.5f + 255.0f * StringConverter::parseReal(xmlElem.attribute("u").value()));
-                            *pChar++ = static_cast<uint8>(0.5f + 255.0f * StringConverter::parseReal(xmlElem.attribute("v").value()));
-                            *pChar++ = static_cast<uint8>(0.5f + 255.0f * StringConverter::parseReal(xmlElem.attribute("w").value()));
-                            *pChar++ = static_cast<uint8>(0.5f + 255.0f * StringConverter::parseReal(xmlElem.attribute("x").value()));
+                            for(int j = 0; j < 4; ++j)
+                            {
+                                // round off instead of just truncating -- avoids magnifying rounding errors
+                                *pChar++ = static_cast<uint8>(0.5f + 255.0f * StringConverter::parseReal(xmlElem.attribute(texcoordDimName[j]).value()));
+                            }
                             break;
-
                         case VET_UBYTE4_NORM:
                             {
                                 elem.baseVertexPointerToElement(pVert, &pCol);
@@ -1214,7 +1165,6 @@ namespace Ogre {
                             OgreAssert(false, "Unsupported VET");
                             break;
                         }
-
                         break;
                     default:
                         break;
